@@ -7,7 +7,7 @@ This module contains all pythran backends.
 from pythran.analyses import LocalNodeDeclarations, GlobalDeclarations, Scope
 from pythran.analyses import YieldPoints, IsAssigned, ASTMatcher, AST_any
 from pythran.analyses import RangeValues, PureExpressions, Dependencies
-from pythran.analyses import Immediates, Ancestors
+from pythran.analyses import Immediates, Ancestors, StrictAliases
 from pythran.config import cfg
 from pythran.cxxgen import Template, Include, Namespace, CompilationUnit
 from pythran.cxxgen import Statement, Block, AnnotatedStatement, Typedef, Label
@@ -156,12 +156,7 @@ def make_function_declaration(self, node, rtype, name, ftypes, fargs,
     arguments = list()
     first_default = len(node.args.args) - len(node.args.defaults)
     for i, (t, a, d) in enumerate(zip(ftypes, fargs, defaults)):
-        # because universal reference and default don't get on well
-        if isinstance(self, CxxGenerator) or i >= first_default:
-            rvalue_ref = ""
-        else:
-            rvalue_ref = "&&"
-        argument = Value(t + rvalue_ref, "{0}{1}".format(a, make_default(d)))
+        argument = Value(t, "{0}{1}".format(a, make_default(d)))
         arguments.append(argument)
     return FunctionDeclaration(Value(rtype, name), arguments, *attributes)
 
@@ -600,9 +595,14 @@ class CxxFunction(ast.NodeVisitor):
         else:
             # For yield function, upper_bound is globals.
             iter_type = ""
-            # Back one step to keep Python behavior (except for break)
-            loop = [If("{} == {}".format(local_iter, upper_bound),
-                    Statement("{} -= {}".format(local_iter, step)))]
+
+            # Back one step to keep Python behavior (unless the loop index does
+            # not escape)
+            if node.target.id in self.scope[node]:
+                loop = []
+            else:
+                loop = [If("{} == {}".format(local_iter, upper_bound),
+                        Statement("{} -= {}".format(local_iter, step)))]
 
         comparison = self.handle_real_loop_comparison(args, local_iter,
                                                       upper_bound)
@@ -708,6 +708,17 @@ class CxxFunction(ast.NodeVisitor):
     def make_assign(self, local_iter_decl, local_iter, iterable):
         return "{0} {1} = {2}".format(local_iter_decl, local_iter, iterable)
 
+    def is_user_function(self, func):
+        aliases = self.strict_aliases[func]
+        if not aliases:
+            return False
+        for alias in aliases:
+            if not isinstance(alias, ast.FunctionDef):
+                return False
+            if self.gather(YieldPoints, alias):
+                return False
+        return True
+
     @cxx_loop
     def visit_For(self, node):
         """
@@ -974,6 +985,10 @@ class CxxFunction(ast.NodeVisitor):
             else:
                 arg = args[0]
             result = fmt.format(attr, arg)
+        # Avoid passing scalars by ref as it prevents some C++ optimization.
+        # pythonic::types::call (tries to) handle that gracefully.
+        elif args and self.is_user_function(node.func):
+            result = "pythonic::types::call({})".format(", ".join([func] + args))
         else:
             result = "{}({})".format(func, ", ".join(args))
 
@@ -1025,8 +1040,25 @@ class CxxFunction(ast.NodeVisitor):
                        for elt in node.elts)
         return self.range_values[node].low >= 0
 
+    def stores_to(self, node):
+        ancestors = self.ancestors[node] + (node,)
+        stmt_indices = [i for i, n in enumerate(ancestors)
+                        if isinstance(n, (ast.Assign, ast.For))]
+        if not stmt_indices:
+            return True
+
+        stmt_index = stmt_indices[-1]
+
+        if isinstance(ancestors[stmt_index], ast.Assign):
+            return ancestors[stmt_index + 1] is ancestors[stmt_index].value
+        else:
+            return ancestors[stmt_index + 1] is not ancestors[stmt_index].target
+
     def visit_Subscript(self, node):
         value = self.visit(node.value)
+        if self.stores_to(node):
+            value = 'pythonic::types::as_const({})'.format(value)
+
         # we cannot overload the [] operator in that case
         if isstr(node.value):
             value = 'pythonic::types::str({})'.format(value)
@@ -1066,12 +1098,19 @@ class CxxFunction(ast.NodeVisitor):
                    else 'pythonic::builtins::None')
             args.append(arg)
 
-        if node.step is None or (isnum(node.step) and node.step.value == 1):
-            if self.all_positive(node.lower) and self.all_positive(node.upper):
-                builder = "pythonic::types::fast_contiguous_slice({},{})"
+        nstep = node.step
+        if nstep is None or (isnum(nstep) and nstep.value > 0):
+            if nstep is None or nstep.value == 1:
+                if self.all_positive(node.lower) and self.all_positive(node.upper):
+                    builder = "pythonic::types::fast_contiguous_slice({0},{1})"
+                else:
+                    builder = "pythonic::types::contiguous_slice({0},{1})"
+                step = 1
             else:
-                builder = "pythonic::types::contiguous_slice({},{})"
-            return builder.format(args[0], args[1])
+                builder = "pythonic::types::cstride_slice<{2}>({0},{1})"
+                step = nstep.value
+
+            return builder.format(args[0], args[1], step)
         else:
             return "pythonic::types::slice({},{},{})".format(*args)
 
@@ -1309,26 +1348,29 @@ class Cxx(Backend):
     >>> print(str(r).replace(os.sep, '/'))
     #include <pythonic/include/types/str.hpp>
     #include <pythonic/types/str.hpp>
-    namespace __pythran_test
+    namespace 
     {
-      struct foo
+      namespace __pythran_test
       {
-        typedef void callable;
-        typedef void pure;
-        struct type
+        struct foo
         {
-          typedef pythonic::types::str __type0;
-          typedef typename pythonic::returnable<__type0>::type __type1;
-          typedef __type1 result_type;
+          typedef void callable;
+          typedef void pure;
+          struct type
+          {
+            typedef pythonic::types::str __type0;
+            typedef typename pythonic::returnable<__type0>::type __type1;
+            typedef __type1 result_type;
+          }  ;
+          inline
+          typename type::result_type operator()() const;
+          ;
         }  ;
         inline
-        typename type::result_type operator()() const;
-        ;
-      }  ;
-      inline
-      typename foo::type::result_type foo::operator()() const
-      {
-        return pythonic::types::str("hello world");
+        typename foo::type::result_type foo::operator()() const
+        {
+          return pythonic::types::str("hello world");
+        }
       }
     }
     """
@@ -1338,7 +1380,7 @@ class Cxx(Backend):
         self.result = None
         super(Cxx, self).__init__(Dependencies, GlobalDeclarations, Types,
                                   Scope, RangeValues, PureExpressions,
-                                  Immediates, Ancestors)
+                                  Immediates, Ancestors, StrictAliases)
 
     # mod
     def visit_Module(self, node):
@@ -1360,7 +1402,8 @@ class Cxx(Backend):
 
         nsbody = [s for ls in decls + defns for s in ls]
         ns = Namespace(pythran_ward + self.passmanager.module_name, nsbody)
-        self.result = CompilationUnit(headers + [ns])
+        anonymous_ns = Namespace("", [ns])  # force internal linkage
+        self.result = CompilationUnit(headers + [anonymous_ns])
 
     def visit_FunctionDef(self, node):
         yields = self.gather(YieldPoints, node)
