@@ -8,8 +8,13 @@ PYTHONIC_NS_BEGIN
 
 namespace numpy
 {
+  template <class E>
+  types::ndarray<long, types::array<long, 1>> argsort(E const &expr,
+                                                      types::none_type);
+
   template <class T, class pS>
-  types::ndarray<long, pS> argsort(types::ndarray<T, pS> const &a);
+  types::ndarray<long, pS> argsort(types::ndarray<T, pS> const &a,
+                                   long axis = -1);
 
   NUMPY_EXPR_TO_NDARRAY0_DECL(argsort);
 
