@@ -9,6 +9,10 @@ PYTHONIC_NS_BEGIN
 
 namespace numpy
 {
+  template <class dtype = functor::float64>
+  typename dtype::type
+  empty(types::pshape<> const &shape, dtype d = dtype());
+
   template <class pS, class dtype = functor::float64>
   types::ndarray<typename dtype::type, sutils::shape_t<pS>>
   empty(pS const &shape, dtype d = dtype());
