@@ -2,59 +2,59 @@
 #define PYTHONIC_INCLUDE_TYPES_NDARRAY_HPP
 
 #include "pythonic/include/types/assignable.hpp"
-#include "pythonic/include/types/empty_iterator.hpp"
 #include "pythonic/include/types/attr.hpp"
+#include "pythonic/include/types/empty_iterator.hpp"
 
-#include "pythonic/include/utils/nested_container.hpp"
-#include "pythonic/include/utils/shared_ref.hpp"
-#include "pythonic/include/utils/reserve.hpp"
-#include "pythonic/include/utils/int_.hpp"
 #include "pythonic/include/utils/broadcast_copy.hpp"
+#include "pythonic/include/utils/int_.hpp"
+#include "pythonic/include/utils/nested_container.hpp"
+#include "pythonic/include/utils/reserve.hpp"
+#include "pythonic/include/utils/shared_ref.hpp"
 
-#include "pythonic/include/types/slice.hpp"
-#include "pythonic/include/types/tuple.hpp"
 #include "pythonic/include/types/list.hpp"
 #include "pythonic/include/types/raw_array.hpp"
+#include "pythonic/include/types/slice.hpp"
+#include "pythonic/include/types/tuple.hpp"
 
 #include "pythonic/include/numpy/bool_.hpp"
-#include "pythonic/include/numpy/uint8.hpp"
-#include "pythonic/include/numpy/int8.hpp"
-#include "pythonic/include/numpy/uint16.hpp"
-#include "pythonic/include/numpy/int16.hpp"
-#include "pythonic/include/numpy/uint32.hpp"
-#include "pythonic/include/numpy/int32.hpp"
-#include "pythonic/include/numpy/uint64.hpp"
-#include "pythonic/include/numpy/int64.hpp"
+#include "pythonic/include/numpy/complex128.hpp"
+#include "pythonic/include/numpy/complex64.hpp"
 #include "pythonic/include/numpy/float32.hpp"
 #include "pythonic/include/numpy/float64.hpp"
-#include "pythonic/include/numpy/complex64.hpp"
-#include "pythonic/include/numpy/complex128.hpp"
+#include "pythonic/include/numpy/int16.hpp"
+#include "pythonic/include/numpy/int32.hpp"
+#include "pythonic/include/numpy/int64.hpp"
+#include "pythonic/include/numpy/int8.hpp"
+#include "pythonic/include/numpy/uint16.hpp"
+#include "pythonic/include/numpy/uint32.hpp"
+#include "pythonic/include/numpy/uint64.hpp"
+#include "pythonic/include/numpy/uint8.hpp"
 
 #include "pythonic/include/types/dynamic_tuple.hpp"
-#include "pythonic/include/types/vectorizable_type.hpp"
-#include "pythonic/include/types/numpy_op_helper.hpp"
 #include "pythonic/include/types/numpy_expr.hpp"
-#include "pythonic/include/types/numpy_texpr.hpp"
-#include "pythonic/include/types/numpy_iexpr.hpp"
 #include "pythonic/include/types/numpy_gexpr.hpp"
+#include "pythonic/include/types/numpy_iexpr.hpp"
+#include "pythonic/include/types/numpy_op_helper.hpp"
+#include "pythonic/include/types/numpy_texpr.hpp"
 #include "pythonic/include/types/numpy_vexpr.hpp"
-#include "pythonic/include/utils/numpy_traits.hpp"
-#include "pythonic/include/utils/array_helper.hpp"
 #include "pythonic/include/types/pointer.hpp"
+#include "pythonic/include/types/vectorizable_type.hpp"
+#include "pythonic/include/utils/array_helper.hpp"
+#include "pythonic/include/utils/numpy_traits.hpp"
 
 #include "pythonic/include/builtins/len.hpp"
 
-#include <cassert>
-#include <ostream>
-#include <iterator>
 #include <array>
+#include <cassert>
 #include <initializer_list>
+#include <iterator>
 #include <numeric>
+#include <ostream>
 
 #ifdef ENABLE_PYTHON_MODULE
-// Cython still uses the deprecated API, so we can't set this macro in this
-// case!
-#ifndef CYTHON_ABI
+// Cython 0.29.x still uses the deprecated API, so we can't set this macro in
+// this case! Also avoid redefining it if already set by the Pythran user.
+#if !defined(CYTHON_ABI) && !defined(NPY_NO_DEPRECATED_API)
 #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #endif
 #include "numpy/arrayobject.h"
@@ -216,6 +216,7 @@ namespace types
   template <class T, class pS>
   struct ndarray {
     static const bool is_vectorizable = types::is_vectorizable<T>::value;
+    static const bool is_flat = true;
     static const bool is_strided = false;
 
     /* types */
@@ -367,14 +368,14 @@ namespace types
     /* element indexing
      * differentiate const from non const, && r-value from l-value
      * */
-    auto fast(long i) const
-        & -> decltype(type_helper<ndarray const &>::get(*this, i))
+    auto fast(
+        long i) const & -> decltype(type_helper<ndarray const &>::get(*this, i))
     {
       return type_helper<ndarray const &>::get(*this, i);
     }
 
-    auto fast(long i) &&
-        -> decltype(type_helper<ndarray>::get(std::move(*this), i))
+    auto fast(long i) && -> decltype(type_helper<ndarray>::get(std::move(*this),
+                                                               i))
     {
       return type_helper<ndarray>::get(std::move(*this), i);
     }
@@ -393,10 +394,10 @@ namespace types
                                                             indices))>::type;
 
     template <class Ty, size_t M>
-        auto fast(array<Ty, M> const &indices) &&
-        -> typename std::enable_if<std::is_integral<Ty>::value,
-                                   decltype(nget<M - 1>().fast(std::move(*this),
-                                                               indices))>::type;
+    auto fast(array<Ty, M> const &indices) && ->
+        typename std::enable_if<std::is_integral<Ty>::value,
+                                decltype(nget<M - 1>().fast(std::move(*this),
+                                                            indices))>::type;
 
 #ifdef USE_XSIMD
     using simd_iterator = const_simd_nditerator<ndarray>;
@@ -432,10 +433,9 @@ namespace types
     operator[](S const &s) const &;
 
     template <class S>
-        typename std::enable_if<is_slice<S>::value,
-                                numpy_gexpr<ndarray, normalize_t<S>>>::type
-        operator[](S const &s) &&
-        ;
+    typename std::enable_if<is_slice<S>::value,
+                            numpy_gexpr<ndarray, normalize_t<S>>>::type
+    operator[](S const &s) &&;
 
     long size() const;
 
@@ -449,19 +449,17 @@ namespace types
     }
 
     template <class S0, class... S>
-    auto operator()(S0 const &s0, S const &... s) const & -> decltype(
+    auto operator()(S0 const &s0, S const &...s) const & -> decltype(
         extended_slice<count_new_axis<S0, S...>::value>{}((*this), s0, s...));
 
     template <class S0, class... S>
-        auto operator()(S0 const &s0, S const &... s) &
-        -> decltype(extended_slice<count_new_axis<S0, S...>::value>{}((*this),
-                                                                      s0,
-                                                                      s...));
+    auto operator()(S0 const &s0, S const &...s) & -> decltype(
+        extended_slice<count_new_axis<S0, S...>::value>{}((*this), s0, s...));
 
     template <class S0, class... S>
-        auto operator()(S0 const &s0, S const &... s) &&
-        -> decltype(extended_slice<count_new_axis<S0, S...>::value>{}(
-            std::move(*this), s0, s...));
+    auto operator()(S0 const &s0, S const &...s) && -> decltype(
+        extended_slice<count_new_axis<S0, S...>::value>{}(std::move(*this), s0,
+                                                          s...));
 
     /* element filtering */
     template <class F> // indexing through an array of boolean -- a mask
@@ -542,10 +540,10 @@ namespace types
                                 decltype(nget<M - 1>()(*this, indices))>::type;
 
     template <class Ty, size_t M>
-        auto operator[](array<Ty, M> const &indices) &&
-        -> typename std::enable_if<std::is_integral<Ty>::value,
-                                   decltype(nget<M - 1>()(std::move(*this),
-                                                          indices))>::type;
+    auto operator[](array<Ty, M> const &indices) && ->
+        typename std::enable_if<std::is_integral<Ty>::value,
+                                decltype(nget<M - 1>()(std::move(*this),
+                                                       indices))>::type;
 
     template <class... Tys, size_t... Is>
     auto _fwdlongindex(std::tuple<Tys...> const &indices,
@@ -597,19 +595,19 @@ namespace types
     }
 
     template <class Slices, size_t... Is>
-    auto _fwdindex(Slices const &indices, utils::index_sequence<Is...>) const
-        & -> decltype((*this)(std::get<Is>(indices)...))
+    auto _fwdindex(Slices const &indices, utils::index_sequence<Is...>)
+        const & -> decltype((*this)(std::get<Is>(indices)...))
     {
       return (*this)(std::get<Is>(indices)...);
     }
 
     template <class S, size_t... Is>
     auto _fwdindex(dynamic_tuple<S> const &indices,
-                   utils::index_sequence<Is...>) const
-        & -> decltype((*this)(std::get<Is>(indices)...))
+                   utils::index_sequence<Is...>)
+        const & -> decltype((*this)(std::get<Is>(indices)...))
     {
       return (*this)((indices.size() > Is ? std::get<Is>(indices)
-                                          : contiguous_slice())...);
+                                          : cstride_slice<1>())...);
     }
 
     template <class Ty0, class Ty1, class... Tys,
@@ -621,8 +619,9 @@ namespace types
                                     indices, utils::make_index_sequence<
                                                  2 + sizeof...(Tys)>()))>::type;
 
-    template <class Ty, size_t M, class _ = typename std::enable_if<
-                                      !std::is_integral<Ty>::value, void>::type>
+    template <class Ty, size_t M,
+              class _ = typename std::enable_if<!std::is_integral<Ty>::value,
+                                                void>::type>
     auto operator[](array<Ty, M> const &indices) const
         & -> decltype(this->_fwdindex(indices, utils::make_index_sequence<M>()))
     {
@@ -648,20 +647,40 @@ namespace types
     flat_iterator fend();
 
     /* member functions */
+    T *data()
+    {
+      return buffer;
+    }
+    T const *data() const
+    {
+      return buffer;
+    }
     long flat_size() const;
     bool may_overlap(ndarray const &) const;
 
     template <class qS>
     ndarray<T, qS> reshape(qS const &shape) const &;
     template <class qS>
-        ndarray<T, qS> reshape(qS const &shape) &&
-        ;
+    ndarray<T, qS> reshape(qS const &shape) &&;
+
+    template <class OT>
+    ndarray<OT, types::array<long, value>> recast()
+    {
+      auto new_shape = sutils::array(_shape);
+      new_shape[value - 1] = new_shape[value - 1] * sizeof(T) / sizeof(OT);
+      auto new_mem = mem.template recast<raw_array<OT>>();
+      return ndarray<OT, types::array<long, value>>(new_mem, new_shape);
+    }
 
     explicit operator bool() const;
 
     ndarray<T, pshape<long>> flat() const;
     ndarray<T, pS> copy() const;
     intptr_t id() const;
+    intptr_t baseid() const
+    {
+      return id();
+    }
     template <size_t I>
     auto shape() const -> decltype(std::get<I>(_shape))
     {
@@ -688,7 +707,7 @@ namespace types
   typename std::enable_if<is_array<E>::value, std::ostream &>::type
   operator<<(std::ostream &os, E const &e);
   /* } */
-}
+} // namespace types
 PYTHONIC_NS_END
 
 /* std::get overloads */
@@ -726,7 +745,13 @@ namespace std
     using type =
         decltype(std::declval<pythonic::types::numpy_gexpr<E, S...>>()[0]);
   };
-}
+
+  template <size_t I, class T, class F>
+  struct tuple_element<I, pythonic::types::numpy_vexpr<T, F>> {
+    using type = decltype(std::declval<pythonic::types::numpy_vexpr<T, F>>()[0]);
+  };
+
+} // namespace std
 
 /* pythran attribute system { */
 #include "pythonic/include/numpy/transpose.hpp"
@@ -751,11 +776,9 @@ namespace types
     struct dtype_helper {
       using table = typename std::conditional<std::is_signed<T>::value,
                                               dtype_table, dtype_utable>::type;
-      using type = typename std::tuple_element <
-                           (sizeof(T) < std::tuple_size<table>::value)
-                       ? sizeof(T)
-                       : 0,
-            table > ::type;
+      using type = typename std::tuple_element<
+          (sizeof(T) < std::tuple_size<table>::value) ? sizeof(T) : 0,
+          table>::type;
     };
 
     template <>
@@ -783,10 +806,10 @@ namespace types
     struct dtype_helper<std::complex<long double>> {
       using type = pythonic::numpy::functor::complex256;
     };
-  }
+  } // namespace details
   template <class T>
   using dtype_t = typename details::dtype_helper<T>::type;
-}
+} // namespace types
 namespace builtins
 {
   namespace details
@@ -794,8 +817,8 @@ namespace builtins
     template <size_t N>
     struct _build_gexpr {
       template <class E, class... S>
-      auto operator()(E const &a, S const &... slices)
-          -> decltype(_build_gexpr<N - 1>{}(a, types::contiguous_slice(),
+      auto operator()(E const &a, S const &...slices)
+          -> decltype(_build_gexpr<N - 1>{}(a, types::cstride_slice<1>(),
                                             slices...));
     };
 
@@ -803,7 +826,7 @@ namespace builtins
     struct _build_gexpr<1> {
       template <class E, class... S>
       types::numpy_gexpr<E, types::normalize_t<S>...>
-      operator()(E const &a, S const &... slices);
+      operator()(E const &a, S const &...slices);
     };
 
     template <class E>
@@ -837,21 +860,36 @@ namespace builtins
     {
       return std::forward<T>(expr)(std::get<Is>(indices)...);
     }
-  }
+  } // namespace details
 
   template <class E>
   types::array<long, E::value> getattr(types::attr::SHAPE, E const &a);
 
+  inline types::pshape<> getattr(types::attr::SHAPE, ...)
+  {
+    return {};
+  }
+
   template <class E>
-  constexpr long getattr(types::attr::NDIM, E const &a);
+  constexpr decltype(long(E::value)) getattr(types::attr::NDIM, E const &a);
 
   template <class E>
   std::integral_constant<long, E::value> getattr(types::attr::NDIM,
                                                  E *const &a);
 
+  inline long getattr(types::attr::NDIM, ...)
+  {
+    return 0;
+  }
+
   template <class E>
   types::array<long, E::value> getattr(types::attr::STRIDES, E const &a);
 
+  inline std::tuple<> getattr(types::attr::STRIDES, ...)
+  {
+    return {};
+  }
+
   template <class E>
   long getattr(types::attr::SIZE, E const &a);
 
@@ -875,8 +913,9 @@ namespace builtins
   }
 
   template <class T, class pS>
-  auto getattr(types::attr::REAL, types::ndarray<T, pS> const &a) -> decltype(
-      details::_make_real(a, utils::int_<types::is_complex<T>::value>{}));
+  auto getattr(types::attr::REAL, types::ndarray<T, pS> const &a)
+      -> decltype(details::_make_real(
+          a, utils::int_<types::is_complex<T>::value>{}));
 
   template <class Op, class... Args>
   auto getattr(types::attr::REAL, types::numpy_expr<Op, Args...> const &a)
@@ -898,20 +937,20 @@ namespace builtins
   }
 
   template <class T, class F>
-  auto getattr(types::attr::REAL, types::numpy_vexpr<T, F> const &a)
-      -> decltype(
-          types::numpy_vexpr<decltype(getattr(types::attr::REAL{}, a.data_)),
-                             F>{getattr(types::attr::REAL{}, a.data_), a.view_})
+  auto
+  getattr(types::attr::REAL, types::numpy_vexpr<T, F> const &a) -> decltype(
+      types::numpy_vexpr<decltype(getattr(types::attr::REAL{}, a.data_)), F>{
+          getattr(types::attr::REAL{}, a.data_), a.view_})
   {
     return {getattr(types::attr::REAL{}, a.data_), a.view_};
   }
 
   template <class E, class... S>
   auto getattr(types::attr::REAL, types::numpy_gexpr<E, S...> const &a)
-      -> decltype(
-          details::real_get(getattr(types::attr::REAL{}, a.arg), a.slices,
-                            utils::make_index_sequence<
-                                std::tuple_size<decltype(a.slices)>::value>()))
+      -> decltype(details::real_get(
+          getattr(types::attr::REAL{}, a.arg), a.slices,
+          utils::make_index_sequence<
+              std::tuple_size<decltype(a.slices)>::value>()))
   {
     return details::real_get(getattr(types::attr::REAL{}, a.arg), a.slices,
                              utils::make_index_sequence<
@@ -919,8 +958,9 @@ namespace builtins
   }
 
   template <class T, class pS>
-  auto getattr(types::attr::IMAG, types::ndarray<T, pS> const &a) -> decltype(
-      details::_make_imag(a, utils::int_<types::is_complex<T>::value>{}));
+  auto getattr(types::attr::IMAG, types::ndarray<T, pS> const &a)
+      -> decltype(details::_make_imag(
+          a, utils::int_<types::is_complex<T>::value>{}));
 
   template <class Op, class... Args>
   auto getattr(types::attr::IMAG, types::numpy_expr<Op, Args...> const &a)
@@ -942,20 +982,20 @@ namespace builtins
   }
 
   template <class T, class F>
-  auto getattr(types::attr::IMAG, types::numpy_vexpr<T, F> const &a)
-      -> decltype(
-          types::numpy_vexpr<decltype(getattr(types::attr::IMAG{}, a.data_)),
-                             F>{getattr(types::attr::IMAG{}, a.data_), a.view_})
+  auto
+  getattr(types::attr::IMAG, types::numpy_vexpr<T, F> const &a) -> decltype(
+      types::numpy_vexpr<decltype(getattr(types::attr::IMAG{}, a.data_)), F>{
+          getattr(types::attr::IMAG{}, a.data_), a.view_})
   {
     return {getattr(types::attr::IMAG{}, a.data_), a.view_};
   }
 
   template <class E, class... S>
   auto getattr(types::attr::IMAG, types::numpy_gexpr<E, S...> const &a)
-      -> decltype(
-          details::imag_get(getattr(types::attr::IMAG{}, a.arg), a.slices,
-                            utils::make_index_sequence<
-                                std::tuple_size<decltype(a.slices)>::value>()))
+      -> decltype(details::imag_get(
+          getattr(types::attr::IMAG{}, a.arg), a.slices,
+          utils::make_index_sequence<
+              std::tuple_size<decltype(a.slices)>::value>()))
   {
     return details::imag_get(getattr(types::attr::IMAG{}, a.arg), a.slices,
                              utils::make_index_sequence<
@@ -965,7 +1005,7 @@ namespace builtins
   template <class E>
   types::dtype_t<typename types::dtype_of<E>::type> getattr(types::attr::DTYPE,
                                                             E const &);
-}
+} // namespace builtins
 PYTHONIC_NS_END
 
 /* } */
@@ -1104,9 +1144,11 @@ namespace std
   template <class T, class pS>
   typename pythonic::types::nditerator<pythonic::types::ndarray<T, pS>> copy(
       typename pythonic::types::const_nditerator<
-          pythonic::types::ndarray<T, pS>> begin,
+          pythonic::types::ndarray<T, pS>>
+          begin,
       typename pythonic::types::const_nditerator<
-          pythonic::types::ndarray<T, pS>> end,
+          pythonic::types::ndarray<T, pS>>
+          end,
       typename pythonic::types::nditerator<pythonic::types::ndarray<T, pS>> out)
   {
     const long offset = pythonic::sutils::prod_tail(begin.data);
@@ -1115,7 +1157,7 @@ namespace std
               out.data.buffer + out.index * offset);
     return out + (end - begin);
   }
-}
+} // namespace std
 
 #endif
 
