@@ -1,16 +1,16 @@
 #ifndef PYTHONIC_NUMPY_RANDOM_GUMBEL_HPP
 #define PYTHONIC_NUMPY_RANDOM_GUMBEL_HPP
 
-#include "pythonic/include/numpy/random/gumbel.hpp"
 #include "pythonic/include/numpy/random/generator.hpp"
+#include "pythonic/include/numpy/random/gumbel.hpp"
 
-#include "pythonic/types/ndarray.hpp"
 #include "pythonic/types/NoneType.hpp"
+#include "pythonic/types/ndarray.hpp"
 #include "pythonic/types/tuple.hpp"
 #include "pythonic/utils/functor.hpp"
 
-#include <random>
 #include <algorithm>
+#include <random>
 
 PYTHONIC_NS_BEGIN
 namespace numpy
@@ -27,13 +27,13 @@ namespace numpy
       return result;
     }
 
-    auto gumbel(double loc, double scale, long size)
+    inline auto gumbel(double loc, double scale, long size)
         -> decltype(gumbel(loc, scale, types::array<long, 1>{{size}}))
     {
       return gumbel(loc, scale, types::array<long, 1>{{size}});
     }
 
-    double gumbel(double loc, double scale, types::none_type d)
+    inline double gumbel(double loc, double scale, types::none_type d)
     {
       double U =
           std::uniform_real_distribution<double>{0., 1.}(details::generator);
@@ -42,8 +42,8 @@ namespace numpy
       }
       return gumbel(loc, scale);
     }
-  }
-}
+  } // namespace random
+} // namespace numpy
 PYTHONIC_NS_END
 
 #endif
