@@ -10,16 +10,16 @@
 #include "pythonic/types/generator.hpp"
 
 PYTHONIC_NS_BEGIN
-yielder::yielder() : __generator_state(0)
+inline yielder::yielder() : __generator_state(0)
 {
 }
 
-bool yielder::operator!=(yielder const &other) const
+inline bool yielder::operator!=(yielder const &other) const
 {
   return __generator_state != other.__generator_state;
 }
 
-bool yielder::operator==(yielder const &other) const
+inline bool yielder::operator==(yielder const &other) const
 {
   return __generator_state == other.__generator_state;
 }
