@@ -3,13 +3,18 @@
 
 #include "pythonic/include/numpy/clip.hpp"
 
-#include "pythonic/utils/functor.hpp"
 #include "pythonic/types/ndarray.hpp"
+#include "pythonic/utils/functor.hpp"
 
 PYTHONIC_NS_BEGIN
 
 namespace numpy
 {
+
+#define NUMPY_NARY_FUNC_NAME _clip_max
+#define NUMPY_NARY_FUNC_SYM xsimd::min
+#include "pythonic/types/numpy_nary_expr.hpp"
+
   namespace wrapper
   {
     template <class T, class Mi, class Ma>
